"use strict";

const Promise = require("bluebird");
const through2 = require("through2");
const assureArray = require("assure-array");

function wrapStreamHandler(stream, handler, callback, item) {
	Promise.try(() => {
		return handler(item);
	}).then((result) => {
		return assureArray(result);
	}).each((newItem) => {
		stream.push(newItem);

		// Silence the "a promise was created in a handler at [...] but was not returned from it" Bluebird warning that occurs in the above; that happens because the `.push` will trigger a *different* wrapped stream's handler (essentially a recursive call), and the Promise that `wrapStreamHandler` creates there is disconnected from this chain. That's not a problem because we use `.nodeify` to wire up the error/completion handling, and so the result of this chain doesn't actually get "lost" like Bluebird thinks.
		return null;
	}).then(() => {
		return;
	}).nodeify(callback);
}

module.exports = function stream(handler, flushHandler) {
	let flushHandlerWrapper = (flushHandler == null)
		? undefined
		: function (callback) {
			wrapStreamHandler(this, flushHandler, callback);
		};
	
	return through2.obj(function (item, _encoding, callback) {
		wrapStreamHandler(this, handler, callback, item);
	}, flushHandlerWrapper);
};
